# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .video_region import VideoRegion

__all__ = ["RoomComposition"]


class RoomComposition(BaseModel):
    id: Optional[str] = None
    """A unique identifier for the room composition."""

    completed_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room composition has completed."""

    created_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room composition was created."""

    download_url: Optional[str] = None
    """Url to download the composition."""

    duration_secs: Optional[int] = None
    """Shows the room composition duration in seconds."""

    ended_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room composition has ended."""

    format: Optional[Literal["mp4"]] = None
    """Shows format of the room composition."""

    record_type: Optional[str] = None

    resolution: Optional[str] = None
    """The resolution of the room composition."""

    room_id: Optional[str] = None
    """Identify the room associated with the room composition."""

    session_id: Optional[str] = None
    """Identify the room session associated with the room composition."""

    size_mb: Optional[float] = None
    """Shows the room composition size in MB."""

    started_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room composition has stated."""

    status: Optional[Literal["completed", "enqueued", "processing"]] = None
    """Shows the room composition status."""

    updated_at: Optional[datetime] = None
    """ISO 8601 timestamp when the room composition was updated."""

    user_id: Optional[str] = None
    """Identify the user associated with the room composition."""

    video_layout: Optional[Dict[str, VideoRegion]] = None
    """Describes the video layout of the room composition in terms of regions.

    Limited to 2 regions.
    """

    webhook_event_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this room composition will be sent if
    sending to the primary URL fails. Must include a scheme, such as 'https'.
    """

    webhook_event_url: Optional[str] = None
    """The URL where webhooks related to this room composition will be sent.

    Must include a scheme, such as 'https'.
    """

    webhook_timeout_secs: Optional[int] = None
    """Specifies how many seconds to wait before timing out a webhook."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["InventoryCoverageListParams", "Filter"]


class InventoryCoverageListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[npa], filter[nxx], filter[administrative_area],
    filter[phone_number_type], filter[country_code], filter[count],
    filter[features], filter[groupBy]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[npa], filter[nxx], filter[administrative_area], filter[phone_number_type], filter[country_code], filter[count], filter[features], filter[groupBy]
    """

    administrative_area: str
    """Filter by administrative area"""

    count: bool
    """Include count in the result"""

    country_code: Literal[
        "AT",
        "AU",
        "BE",
        "BG",
        "CA",
        "CH",
        "CN",
        "CY",
        "CZ",
        "DE",
        "DK",
        "EE",
        "ES",
        "FI",
        "FR",
        "GB",
        "GR",
        "HU",
        "HR",
        "IE",
        "IT",
        "LT",
        "LU",
        "LV",
        "NL",
        "NZ",
        "MX",
        "NO",
        "PL",
        "PT",
        "RO",
        "SE",
        "SG",
        "SI",
        "SK",
        "US",
    ]
    """Filter by country. Defaults to US"""

    features: List[Literal["sms", "mms", "voice", "fax", "emergency"]]
    """Filter if the phone number should be used for voice, fax, mms, sms, emergency.

    Returns features in the response when used.
    """

    group_by: Annotated[Literal["locality", "npa", "national_destination_code"], PropertyInfo(alias="groupBy")]
    """Filter to group results"""

    npa: int
    """Filter by npa"""

    nxx: int
    """Filter by nxx"""

    phone_number_type: Literal["local", "toll_free", "national", "mobile", "landline", "shared_cost"]
    """Filter by phone number type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_streaming_started import CallStreamingStarted

__all__ = ["CallStreamingStartedWebhookEvent"]


class CallStreamingStartedWebhookEvent(BaseModel):
    data: Optional[CallStreamingStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.session_analysis.metadata_retrieve_response import MetadataRetrieveResponse
from ...types.session_analysis.metadata_retrieve_record_type_response import MetadataRetrieveRecordTypeResponse

__all__ = ["MetadataResource", "AsyncMetadataResource"]


class MetadataResource(SyncAPIResource):
    """Analyze voice AI sessions, costs, and event hierarchies across Telnyx products."""

    @cached_property
    def with_raw_response(self) -> MetadataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MetadataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MetadataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MetadataResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MetadataRetrieveResponse:
        """
        Returns all available record types and supported query parameters for session
        analysis.
        """
        return self._get(
            "/session_analysis/metadata",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MetadataRetrieveResponse,
        )

    def retrieve_record_type(
        self,
        record_type: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MetadataRetrieveRecordTypeResponse:
        """
        Returns detailed metadata for a specific record type, including relationships
        and examples.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not record_type:
            raise ValueError(f"Expected a non-empty value for `record_type` but received {record_type!r}")
        return self._get(
            f"/session_analysis/metadata/{record_type}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MetadataRetrieveRecordTypeResponse,
        )


class AsyncMetadataResource(AsyncAPIResource):
    """Analyze voice AI sessions, costs, and event hierarchies across Telnyx products."""

    @cached_property
    def with_raw_response(self) -> AsyncMetadataResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMetadataResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMetadataResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMetadataResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MetadataRetrieveResponse:
        """
        Returns all available record types and supported query parameters for session
        analysis.
        """
        return await self._get(
            "/session_analysis/metadata",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MetadataRetrieveResponse,
        )

    async def retrieve_record_type(
        self,
        record_type: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MetadataRetrieveRecordTypeResponse:
        """
        Returns detailed metadata for a specific record type, including relationships
        and examples.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not record_type:
            raise ValueError(f"Expected a non-empty value for `record_type` but received {record_type!r}")
        return await self._get(
            f"/session_analysis/metadata/{record_type}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MetadataRetrieveRecordTypeResponse,
        )


class MetadataResourceWithRawResponse:
    def __init__(self, metadata: MetadataResource) -> None:
        self._metadata = metadata

        self.retrieve = to_raw_response_wrapper(
            metadata.retrieve,
        )
        self.retrieve_record_type = to_raw_response_wrapper(
            metadata.retrieve_record_type,
        )


class AsyncMetadataResourceWithRawResponse:
    def __init__(self, metadata: AsyncMetadataResource) -> None:
        self._metadata = metadata

        self.retrieve = async_to_raw_response_wrapper(
            metadata.retrieve,
        )
        self.retrieve_record_type = async_to_raw_response_wrapper(
            metadata.retrieve_record_type,
        )


class MetadataResourceWithStreamingResponse:
    def __init__(self, metadata: MetadataResource) -> None:
        self._metadata = metadata

        self.retrieve = to_streamed_response_wrapper(
            metadata.retrieve,
        )
        self.retrieve_record_type = to_streamed_response_wrapper(
            metadata.retrieve_record_type,
        )


class AsyncMetadataResourceWithStreamingResponse:
    def __init__(self, metadata: AsyncMetadataResource) -> None:
        self._metadata = metadata

        self.retrieve = async_to_streamed_response_wrapper(
            metadata.retrieve,
        )
        self.retrieve_record_type = async_to_streamed_response_wrapper(
            metadata.retrieve_record_type,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PhoneNumberListParams", "Filter", "FilterCivicAddressID", "FilterLocationID", "FilterPhoneNumber"]


class PhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Filter parameter for phone numbers (deepObject style).

    Supports filtering by phone_number, civic_address_id, and location_id with
    eq/contains operations.
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterCivicAddressID(TypedDict, total=False):
    eq: str
    """The civic address ID to filter by"""


class FilterLocationID(TypedDict, total=False):
    eq: str
    """The location ID to filter by"""


class FilterPhoneNumber(TypedDict, total=False):
    contains: str
    """The phone number to filter by (partial match)"""

    eq: str
    """The phone number to filter by (exact match)"""


class Filter(TypedDict, total=False):
    """Filter parameter for phone numbers (deepObject style).

    Supports filtering by phone_number, civic_address_id, and location_id with eq/contains operations.
    """

    civic_address_id: FilterCivicAddressID

    location_id: FilterLocationID

    phone_number: FilterPhoneNumber



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingPhoneNumberBlock", "ActivationRange", "PhoneNumberRange"]


class ActivationRange(BaseModel):
    end_at: Optional[str] = None
    """Specifies the end of the activation range.

    It must be no more than the end of the phone number range.
    """

    start_at: Optional[str] = None
    """Specifies the start of the activation range.

    Must be greater or equal the start of the phone number range.
    """


class PhoneNumberRange(BaseModel):
    """Specifies the phone number range for this porting phone number block."""

    end_at: Optional[str] = None
    """
    Specifies the end of the phone number range for this porting phone number block.
    """

    start_at: Optional[str] = None
    """
    Specifies the start of the phone number range for this porting phone number
    block.
    """


class PortingPhoneNumberBlock(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this porting phone number block."""

    activation_ranges: Optional[List[ActivationRange]] = None
    """Specifies the activation ranges for this porting phone number block.

    The activation range must be within the phone number range and should not
    overlap with other activation ranges.
    """

    country_code: Optional[str] = None
    """Specifies the country code for this porting phone number block.

    It is a two-letter ISO 3166-1 alpha-2 country code.
    """

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    phone_number_range: Optional[PhoneNumberRange] = None
    """Specifies the phone number range for this porting phone number block."""

    phone_number_type: Optional[Literal["landline", "local", "mobile", "national", "shared_cost", "toll_free"]] = None
    """Specifies the phone number type for this porting phone number block."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..wireless_error import WirelessError
from ..shared.simple_sim_card import SimpleSimCard

__all__ = ["RegisterCreateResponse"]


class RegisterCreateResponse(BaseModel):
    data: Optional[List[SimpleSimCard]] = None
    """Successfully registered SIM cards."""

    errors: Optional[List[WirelessError]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .storage import (
    StorageResource,
    AsyncStorageResource,
    StorageResourceWithRawResponse,
    AsyncStorageResourceWithRawResponse,
    StorageResourceWithStreamingResponse,
    AsyncStorageResourceWithStreamingResponse,
)
from .migrations import (
    MigrationsResource,
    AsyncMigrationsResource,
    MigrationsResourceWithRawResponse,
    AsyncMigrationsResourceWithRawResponse,
    MigrationsResourceWithStreamingResponse,
    AsyncMigrationsResourceWithStreamingResponse,
)
from .migration_sources import (
    MigrationSourcesResource,
    AsyncMigrationSourcesResource,
    MigrationSourcesResourceWithRawResponse,
    AsyncMigrationSourcesResourceWithRawResponse,
    MigrationSourcesResourceWithStreamingResponse,
    AsyncMigrationSourcesResourceWithStreamingResponse,
)

__all__ = [
    "BucketsResource",
    "AsyncBucketsResource",
    "BucketsResourceWithRawResponse",
    "AsyncBucketsResourceWithRawResponse",
    "BucketsResourceWithStreamingResponse",
    "AsyncBucketsResourceWithStreamingResponse",
    "MigrationSourcesResource",
    "AsyncMigrationSourcesResource",
    "MigrationSourcesResourceWithRawResponse",
    "AsyncMigrationSourcesResourceWithRawResponse",
    "MigrationSourcesResourceWithStreamingResponse",
    "AsyncMigrationSourcesResourceWithStreamingResponse",
    "MigrationsResource",
    "AsyncMigrationsResource",
    "MigrationsResourceWithRawResponse",
    "AsyncMigrationsResourceWithRawResponse",
    "MigrationsResourceWithStreamingResponse",
    "AsyncMigrationsResourceWithStreamingResponse",
    "StorageResource",
    "AsyncStorageResource",
    "StorageResourceWithRawResponse",
    "AsyncStorageResourceWithRawResponse",
    "StorageResourceWithStreamingResponse",
    "AsyncStorageResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import Fax, FaxCreateResponse, FaxRetrieveResponse
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFaxes:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        fax = client.faxes.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        )
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        fax = client.faxes.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
            black_threshold=1,
            client_state="aGF2ZSBhIG5pY2UgZGF5ID1d",
            from_display_name="Company Name",
            media_name="my_media_uploaded_to_media_storage_api",
            media_url="https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf",
            monochrome=True,
            preview_format="pdf",
            quality="high",
            store_media=True,
            store_preview=True,
            t38_enabled=True,
            webhook_url="https://www.example.com/server-b/",
        )
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.faxes.with_raw_response.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = response.parse()
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.faxes.with_streaming_response.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = response.parse()
            assert_matches_type(FaxCreateResponse, fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        fax = client.faxes.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.faxes.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = response.parse()
        assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.faxes.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = response.parse()
            assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.faxes.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        fax = client.faxes.list()
        assert_matches_type(SyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        fax = client.faxes.list(
            filter={
                "created_at": {
                    "gt": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "gte": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "lt": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "lte": parse_datetime("2020-02-02T22:25:27.521992Z"),
                },
                "direction": {"eq": "inbound"},
                "from": {"eq": "+13127367276"},
                "to": {"eq": "+13127367276"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.faxes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.faxes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[Fax], fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        fax = client.faxes.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert fax is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.faxes.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = response.parse()
        assert fax is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.faxes.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = response.parse()
            assert fax is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.faxes.with_raw_response.delete(
                "",
            )


class TestAsyncFaxes:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        )
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
            black_threshold=1,
            client_state="aGF2ZSBhIG5pY2UgZGF5ID1d",
            from_display_name="Company Name",
            media_name="my_media_uploaded_to_media_storage_api",
            media_url="https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf",
            monochrome=True,
            preview_format="pdf",
            quality="high",
            store_media=True,
            store_preview=True,
            t38_enabled=True,
            webhook_url="https://www.example.com/server-b/",
        )
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.faxes.with_raw_response.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = await response.parse()
        assert_matches_type(FaxCreateResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.faxes.with_streaming_response.create(
            connection_id="234423",
            from_="+13125790015",
            to="+13127367276",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = await response.parse()
            assert_matches_type(FaxCreateResponse, fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.faxes.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = await response.parse()
        assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.faxes.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = await response.parse()
            assert_matches_type(FaxRetrieveResponse, fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.faxes.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.list()
        assert_matches_type(AsyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.list(
            filter={
                "created_at": {
                    "gt": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "gte": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "lt": parse_datetime("2020-02-02T22:25:27.521992Z"),
                    "lte": parse_datetime("2020-02-02T22:25:27.521992Z"),
                },
                "direction": {"eq": "inbound"},
                "from": {"eq": "+13127367276"},
                "to": {"eq": "+13127367276"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.faxes.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[Fax], fax, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.faxes.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[Fax], fax, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        fax = await async_client.faxes.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert fax is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.faxes.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        fax = await response.parse()
        assert fax is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.faxes.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            fax = await response.parse()
            assert fax is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.faxes.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .hangup_tool_params import HangupToolParams

__all__ = ["HangupTool"]


class HangupTool(BaseModel):
    hangup: HangupToolParams

    type: Literal["hangup"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date, datetime

from ....._models import BaseModel
from .telco_data_usage_record import TelcoDataUsageRecord

__all__ = ["TelcoDataUsageReportResponse"]


class TelcoDataUsageReportResponse(BaseModel):
    """Telco data usage report response"""

    id: Optional[str] = None
    """Unique identifier for the report"""

    aggregation_type: Optional[str] = None
    """Type of aggregation used in the report"""

    created_at: Optional[datetime] = None
    """Timestamp when the report was created"""

    end_date: Optional[date] = None
    """End date of the report period"""

    managed_accounts: Optional[List[str]] = None
    """List of managed account IDs included in the report"""

    record_type: Optional[str] = None
    """Record type identifier"""

    report_url: Optional[str] = None
    """URL to download the complete report"""

    result: Optional[List[TelcoDataUsageRecord]] = None
    """Array of usage records"""

    start_date: Optional[date] = None
    """Start date of the report period"""

    status: Optional[str] = None
    """Current status of the report"""

    updated_at: Optional[datetime] = None
    """Timestamp when the report was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .insights import (
    InsightsResource,
    AsyncInsightsResource,
    InsightsResourceWithRawResponse,
    AsyncInsightsResourceWithRawResponse,
    InsightsResourceWithStreamingResponse,
    AsyncInsightsResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)
from .insight_groups import (
    InsightGroupsResource,
    AsyncInsightGroupsResource,
    InsightGroupsResourceWithRawResponse,
    AsyncInsightGroupsResourceWithRawResponse,
    InsightGroupsResourceWithStreamingResponse,
    AsyncInsightGroupsResourceWithStreamingResponse,
)

__all__ = [
    "InsightGroupsResource",
    "AsyncInsightGroupsResource",
    "InsightGroupsResourceWithRawResponse",
    "AsyncInsightGroupsResourceWithRawResponse",
    "InsightGroupsResourceWithStreamingResponse",
    "AsyncInsightGroupsResourceWithStreamingResponse",
    "InsightsResource",
    "AsyncInsightsResource",
    "InsightsResourceWithRawResponse",
    "AsyncInsightsResourceWithRawResponse",
    "InsightsResourceWithStreamingResponse",
    "AsyncInsightsResourceWithStreamingResponse",
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "ConversationsResource",
    "AsyncConversationsResource",
    "ConversationsResourceWithRawResponse",
    "AsyncConversationsResourceWithRawResponse",
    "ConversationsResourceWithStreamingResponse",
    "AsyncConversationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel
from .webhook_portout_new_comment import WebhookPortoutNewComment
from .webhook_portout_status_changed import WebhookPortoutStatusChanged
from .webhook_portout_foc_date_changed import WebhookPortoutFocDateChanged

__all__ = ["EventRetrieveResponse", "Data"]

Data: TypeAlias = Union[WebhookPortoutStatusChanged, WebhookPortoutNewComment, WebhookPortoutFocDateChanged]


class EventRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CallControlApplicationInbound"]


class CallControlApplicationInbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the total number of inbound calls to phone numbers
    associated with this connection.
    """

    shaken_stir_enabled: Optional[bool] = None
    """
    When enabled Telnyx will include Shaken/Stir data in the Webhook for new inbound
    calls.
    """

    sip_subdomain: Optional[str] = None
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Optional[Literal["only_my_connections", "from_anyone"]] = None
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import advanced_order_create_params, advanced_order_update_requirement_group_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.advanced_order_list_response import AdvancedOrderListResponse
from ..types.advanced_order_create_response import AdvancedOrderCreateResponse
from ..types.advanced_order_retrieve_response import AdvancedOrderRetrieveResponse
from ..types.advanced_order_update_requirement_group_response import AdvancedOrderUpdateRequirementGroupResponse

__all__ = ["AdvancedOrdersResource", "AsyncAdvancedOrdersResource"]


class AdvancedOrdersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AdvancedOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AdvancedOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdvancedOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AdvancedOrdersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        area_code: str | Omit = omit,
        comments: str | Omit = omit,
        country_code: str | Omit = omit,
        customer_reference: str | Omit = omit,
        features: List[Literal["sms", "mms", "voice", "fax", "emergency"]] | Omit = omit,
        phone_number_type: Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"] | Omit = omit,
        quantity: int | Omit = omit,
        requirement_group_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderCreateResponse:
        """
        Create Advanced Order

        Args:
          requirement_group_id: The ID of the requirement group to associate with this advanced order

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/advanced_orders",
            body=maybe_transform(
                {
                    "area_code": area_code,
                    "comments": comments,
                    "country_code": country_code,
                    "customer_reference": customer_reference,
                    "features": features,
                    "phone_number_type": phone_number_type,
                    "quantity": quantity,
                    "requirement_group_id": requirement_group_id,
                },
                advanced_order_create_params.AdvancedOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderCreateResponse,
        )

    def retrieve(
        self,
        order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderRetrieveResponse:
        """
        Get Advanced Order

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not order_id:
            raise ValueError(f"Expected a non-empty value for `order_id` but received {order_id!r}")
        return self._get(
            f"/advanced_orders/{order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderListResponse:
        """List Advanced Orders"""
        return self._get(
            "/advanced_orders",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderListResponse,
        )

    def update_requirement_group(
        self,
        advanced_order_id: str,
        *,
        area_code: str | Omit = omit,
        comments: str | Omit = omit,
        country_code: str | Omit = omit,
        customer_reference: str | Omit = omit,
        features: List[Literal["sms", "mms", "voice", "fax", "emergency"]] | Omit = omit,
        phone_number_type: Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"] | Omit = omit,
        quantity: int | Omit = omit,
        requirement_group_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderUpdateRequirementGroupResponse:
        """
        Update Advanced Order

        Args:
          requirement_group_id: The ID of the requirement group to associate with this advanced order

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not advanced_order_id:
            raise ValueError(f"Expected a non-empty value for `advanced_order_id` but received {advanced_order_id!r}")
        return self._patch(
            f"/advanced_orders/{advanced_order_id}/requirement_group",
            body=maybe_transform(
                {
                    "area_code": area_code,
                    "comments": comments,
                    "country_code": country_code,
                    "customer_reference": customer_reference,
                    "features": features,
                    "phone_number_type": phone_number_type,
                    "quantity": quantity,
                    "requirement_group_id": requirement_group_id,
                },
                advanced_order_update_requirement_group_params.AdvancedOrderUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderUpdateRequirementGroupResponse,
        )


class AsyncAdvancedOrdersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAdvancedOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAdvancedOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdvancedOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAdvancedOrdersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        area_code: str | Omit = omit,
        comments: str | Omit = omit,
        country_code: str | Omit = omit,
        customer_reference: str | Omit = omit,
        features: List[Literal["sms", "mms", "voice", "fax", "emergency"]] | Omit = omit,
        phone_number_type: Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"] | Omit = omit,
        quantity: int | Omit = omit,
        requirement_group_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderCreateResponse:
        """
        Create Advanced Order

        Args:
          requirement_group_id: The ID of the requirement group to associate with this advanced order

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/advanced_orders",
            body=await async_maybe_transform(
                {
                    "area_code": area_code,
                    "comments": comments,
                    "country_code": country_code,
                    "customer_reference": customer_reference,
                    "features": features,
                    "phone_number_type": phone_number_type,
                    "quantity": quantity,
                    "requirement_group_id": requirement_group_id,
                },
                advanced_order_create_params.AdvancedOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderCreateResponse,
        )

    async def retrieve(
        self,
        order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderRetrieveResponse:
        """
        Get Advanced Order

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not order_id:
            raise ValueError(f"Expected a non-empty value for `order_id` but received {order_id!r}")
        return await self._get(
            f"/advanced_orders/{order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderListResponse:
        """List Advanced Orders"""
        return await self._get(
            "/advanced_orders",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderListResponse,
        )

    async def update_requirement_group(
        self,
        advanced_order_id: str,
        *,
        area_code: str | Omit = omit,
        comments: str | Omit = omit,
        country_code: str | Omit = omit,
        customer_reference: str | Omit = omit,
        features: List[Literal["sms", "mms", "voice", "fax", "emergency"]] | Omit = omit,
        phone_number_type: Literal["local", "mobile", "toll_free", "shared_cost", "national", "landline"] | Omit = omit,
        quantity: int | Omit = omit,
        requirement_group_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdvancedOrderUpdateRequirementGroupResponse:
        """
        Update Advanced Order

        Args:
          requirement_group_id: The ID of the requirement group to associate with this advanced order

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not advanced_order_id:
            raise ValueError(f"Expected a non-empty value for `advanced_order_id` but received {advanced_order_id!r}")
        return await self._patch(
            f"/advanced_orders/{advanced_order_id}/requirement_group",
            body=await async_maybe_transform(
                {
                    "area_code": area_code,
                    "comments": comments,
                    "country_code": country_code,
                    "customer_reference": customer_reference,
                    "features": features,
                    "phone_number_type": phone_number_type,
                    "quantity": quantity,
                    "requirement_group_id": requirement_group_id,
                },
                advanced_order_update_requirement_group_params.AdvancedOrderUpdateRequirementGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdvancedOrderUpdateRequirementGroupResponse,
        )


class AdvancedOrdersResourceWithRawResponse:
    def __init__(self, advanced_orders: AdvancedOrdersResource) -> None:
        self._advanced_orders = advanced_orders

        self.create = to_raw_response_wrapper(
            advanced_orders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            advanced_orders.retrieve,
        )
        self.list = to_raw_response_wrapper(
            advanced_orders.list,
        )
        self.update_requirement_group = to_raw_response_wrapper(
            advanced_orders.update_requirement_group,
        )


class AsyncAdvancedOrdersResourceWithRawResponse:
    def __init__(self, advanced_orders: AsyncAdvancedOrdersResource) -> None:
        self._advanced_orders = advanced_orders

        self.create = async_to_raw_response_wrapper(
            advanced_orders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            advanced_orders.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            advanced_orders.list,
        )
        self.update_requirement_group = async_to_raw_response_wrapper(
            advanced_orders.update_requirement_group,
        )


class AdvancedOrdersResourceWithStreamingResponse:
    def __init__(self, advanced_orders: AdvancedOrdersResource) -> None:
        self._advanced_orders = advanced_orders

        self.create = to_streamed_response_wrapper(
            advanced_orders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            advanced_orders.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            advanced_orders.list,
        )
        self.update_requirement_group = to_streamed_response_wrapper(
            advanced_orders.update_requirement_group,
        )


class AsyncAdvancedOrdersResourceWithStreamingResponse:
    def __init__(self, advanced_orders: AsyncAdvancedOrdersResource) -> None:
        self._advanced_orders = advanced_orders

        self.create = async_to_streamed_response_wrapper(
            advanced_orders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            advanced_orders.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            advanced_orders.list,
        )
        self.update_requirement_group = async_to_streamed_response_wrapper(
            advanced_orders.update_requirement_group,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .portout_comment import PortoutComment

__all__ = ["CommentCreateResponse"]


class CommentCreateResponse(BaseModel):
    data: Optional[PortoutComment] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SiprecConnectorCreateResponse,
    SiprecConnectorUpdateResponse,
    SiprecConnectorRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSiprecConnectors:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
            app_subdomain="my-app",
        )
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.siprec_connectors.with_raw_response.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = response.parse()
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.siprec_connectors.with_streaming_response.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = response.parse()
            assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.retrieve(
            "connector_name",
        )
        assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.siprec_connectors.with_raw_response.retrieve(
            "connector_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = response.parse()
        assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.siprec_connectors.with_streaming_response.retrieve(
            "connector_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = response.parse()
            assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            client.siprec_connectors.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
            app_subdomain="my-app",
        )
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.siprec_connectors.with_raw_response.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = response.parse()
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.siprec_connectors.with_streaming_response.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = response.parse()
            assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            client.siprec_connectors.with_raw_response.update(
                connector_name="",
                host="siprec.telnyx.com",
                name="my-siprec-connector",
                port=5060,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        siprec_connector = client.siprec_connectors.delete(
            "connector_name",
        )
        assert siprec_connector is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.siprec_connectors.with_raw_response.delete(
            "connector_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = response.parse()
        assert siprec_connector is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.siprec_connectors.with_streaming_response.delete(
            "connector_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = response.parse()
            assert siprec_connector is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            client.siprec_connectors.with_raw_response.delete(
                "",
            )


class TestAsyncSiprecConnectors:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
            app_subdomain="my-app",
        )
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.siprec_connectors.with_raw_response.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = await response.parse()
        assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.siprec_connectors.with_streaming_response.create(
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = await response.parse()
            assert_matches_type(SiprecConnectorCreateResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.retrieve(
            "connector_name",
        )
        assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.siprec_connectors.with_raw_response.retrieve(
            "connector_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = await response.parse()
        assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.siprec_connectors.with_streaming_response.retrieve(
            "connector_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = await response.parse()
            assert_matches_type(SiprecConnectorRetrieveResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            await async_client.siprec_connectors.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
            app_subdomain="my-app",
        )
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.siprec_connectors.with_raw_response.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = await response.parse()
        assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.siprec_connectors.with_streaming_response.update(
            connector_name="connector_name",
            host="siprec.telnyx.com",
            name="my-siprec-connector",
            port=5060,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = await response.parse()
            assert_matches_type(SiprecConnectorUpdateResponse, siprec_connector, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            await async_client.siprec_connectors.with_raw_response.update(
                connector_name="",
                host="siprec.telnyx.com",
                name="my-siprec-connector",
                port=5060,
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        siprec_connector = await async_client.siprec_connectors.delete(
            "connector_name",
        )
        assert siprec_connector is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.siprec_connectors.with_raw_response.delete(
            "connector_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        siprec_connector = await response.parse()
        assert siprec_connector is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.siprec_connectors.with_streaming_response.delete(
            "connector_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            siprec_connector = await response.parse()
            assert siprec_connector is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connector_name` but received ''"):
            await async_client.siprec_connectors.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MessagingHostedNumberOrderCheckEligibilityResponse", "PhoneNumber"]


class PhoneNumber(BaseModel):
    detail: Optional[str] = None
    """Detailed information about the eligibility status."""

    eligible: Optional[bool] = None
    """Whether the phone number is eligible for hosted messaging."""

    eligible_status: Optional[
        Literal[
            "NUMBER_CAN_NOT_BE_REPEATED",
            "NUMBER_CAN_NOT_BE_VALIDATED",
            "NUMBER_CAN_NOT_BE_WIRELESS",
            "NUMBER_CAN_NOT_BE_ACTIVE_IN_YOUR_ACCOUNT",
            "NUMBER_CAN_NOT_HOSTED_WITH_A_TELNYX_SUBSCRIBER",
            "NUMBER_CAN_NOT_BE_IN_TELNYX",
            "NUMBER_IS_NOT_A_US_NUMBER",
            "NUMBER_IS_NOT_A_VALID_ROUTING_NUMBER",
            "NUMBER_IS_NOT_IN_E164_FORMAT",
            "BILLING_ACCOUNT_CHECK_FAILED",
            "BILLING_ACCOUNT_IS_ABOLISHED",
            "ELIGIBLE",
        ]
    ] = None
    """The eligibility status of the phone number."""

    phone_number: Optional[str] = None
    """The phone number in e164 format."""


class MessagingHostedNumberOrderCheckEligibilityResponse(BaseModel):
    phone_numbers: Optional[List[PhoneNumber]] = None
    """List of phone numbers with their eligibility status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import (
    sim_card_group_list_params,
    sim_card_group_create_params,
    sim_card_group_update_params,
    sim_card_group_retrieve_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.sim_card_group_list_response import SimCardGroupListResponse
from ...types.sim_card_group_create_response import SimCardGroupCreateResponse
from ...types.sim_card_group_delete_response import SimCardGroupDeleteResponse
from ...types.sim_card_group_update_response import SimCardGroupUpdateResponse
from ...types.sim_card_group_retrieve_response import SimCardGroupRetrieveResponse

__all__ = ["SimCardGroupsResource", "AsyncSimCardGroupsResource"]


class SimCardGroupsResource(SyncAPIResource):
    """SIM Card Groups operations"""

    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SimCardGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SimCardGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimCardGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SimCardGroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        data_limit: sim_card_group_create_params.DataLimit | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupCreateResponse:
        """
        Creates a new SIM card group object

        Args:
          name: A user friendly name for the SIM card group.

          data_limit: Upper limit on the amount of data the SIM cards, within the group, can use.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_card_groups",
            body=maybe_transform(
                {
                    "name": name,
                    "data_limit": data_limit,
                },
                sim_card_group_create_params.SimCardGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        include_iccids: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupRetrieveResponse:
        """
        Returns the details regarding a specific SIM card group

        Args:
          include_iccids: It includes a list of associated ICCIDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_card_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_iccids": include_iccids}, sim_card_group_retrieve_params.SimCardGroupRetrieveParams
                ),
            ),
            cast_to=SimCardGroupRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        data_limit: sim_card_group_update_params.DataLimit | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupUpdateResponse:
        """
        Updates a SIM card group

        Args:
          data_limit: Upper limit on the amount of data the SIM cards, within the group, can use.

          name: A user friendly name for the SIM card group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/sim_card_groups/{id}",
            body=maybe_transform(
                {
                    "data_limit": data_limit,
                    "name": name,
                },
                sim_card_group_update_params.SimCardGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupUpdateResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        filter_private_wireless_gateway_id: str | Omit = omit,
        filter_wireless_blocklist_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[SimCardGroupListResponse]:
        """
        Get all SIM card groups belonging to the user that match the given filters.

        Args:
          filter_name: A valid SIM card group name.

          filter_private_wireless_gateway_id: A Private Wireless Gateway ID associated with the group.

          filter_wireless_blocklist_id: A Wireless Blocklist ID associated with the group.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_groups",
            page=SyncDefaultFlatPagination[SimCardGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "filter_private_wireless_gateway_id": filter_private_wireless_gateway_id,
                        "filter_wireless_blocklist_id": filter_wireless_blocklist_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_group_list_params.SimCardGroupListParams,
                ),
            ),
            model=SimCardGroupListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupDeleteResponse:
        """
        Permanently deletes a SIM card group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/sim_card_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupDeleteResponse,
        )


class AsyncSimCardGroupsResource(AsyncAPIResource):
    """SIM Card Groups operations"""

    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSimCardGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimCardGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimCardGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSimCardGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        data_limit: sim_card_group_create_params.DataLimit | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupCreateResponse:
        """
        Creates a new SIM card group object

        Args:
          name: A user friendly name for the SIM card group.

          data_limit: Upper limit on the amount of data the SIM cards, within the group, can use.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_card_groups",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "data_limit": data_limit,
                },
                sim_card_group_create_params.SimCardGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        include_iccids: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupRetrieveResponse:
        """
        Returns the details regarding a specific SIM card group

        Args:
          include_iccids: It includes a list of associated ICCIDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_card_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_iccids": include_iccids}, sim_card_group_retrieve_params.SimCardGroupRetrieveParams
                ),
            ),
            cast_to=SimCardGroupRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        data_limit: sim_card_group_update_params.DataLimit | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupUpdateResponse:
        """
        Updates a SIM card group

        Args:
          data_limit: Upper limit on the amount of data the SIM cards, within the group, can use.

          name: A user friendly name for the SIM card group.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/sim_card_groups/{id}",
            body=await async_maybe_transform(
                {
                    "data_limit": data_limit,
                    "name": name,
                },
                sim_card_group_update_params.SimCardGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupUpdateResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        filter_private_wireless_gateway_id: str | Omit = omit,
        filter_wireless_blocklist_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimCardGroupListResponse, AsyncDefaultFlatPagination[SimCardGroupListResponse]]:
        """
        Get all SIM card groups belonging to the user that match the given filters.

        Args:
          filter_name: A valid SIM card group name.

          filter_private_wireless_gateway_id: A Private Wireless Gateway ID associated with the group.

          filter_wireless_blocklist_id: A Wireless Blocklist ID associated with the group.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_groups",
            page=AsyncDefaultFlatPagination[SimCardGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "filter_private_wireless_gateway_id": filter_private_wireless_gateway_id,
                        "filter_wireless_blocklist_id": filter_wireless_blocklist_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    sim_card_group_list_params.SimCardGroupListParams,
                ),
            ),
            model=SimCardGroupListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SimCardGroupDeleteResponse:
        """
        Permanently deletes a SIM card group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/sim_card_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SimCardGroupDeleteResponse,
        )


class SimCardGroupsResourceWithRawResponse:
    def __init__(self, sim_card_groups: SimCardGroupsResource) -> None:
        self._sim_card_groups = sim_card_groups

        self.create = to_raw_response_wrapper(
            sim_card_groups.create,
        )
        self.retrieve = to_raw_response_wrapper(
            sim_card_groups.retrieve,
        )
        self.update = to_raw_response_wrapper(
            sim_card_groups.update,
        )
        self.list = to_raw_response_wrapper(
            sim_card_groups.list,
        )
        self.delete = to_raw_response_wrapper(
            sim_card_groups.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._sim_card_groups.actions)


class AsyncSimCardGroupsResourceWithRawResponse:
    def __init__(self, sim_card_groups: AsyncSimCardGroupsResource) -> None:
        self._sim_card_groups = sim_card_groups

        self.create = async_to_raw_response_wrapper(
            sim_card_groups.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            sim_card_groups.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            sim_card_groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            sim_card_groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sim_card_groups.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._sim_card_groups.actions)


class SimCardGroupsResourceWithStreamingResponse:
    def __init__(self, sim_card_groups: SimCardGroupsResource) -> None:
        self._sim_card_groups = sim_card_groups

        self.create = to_streamed_response_wrapper(
            sim_card_groups.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            sim_card_groups.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            sim_card_groups.update,
        )
        self.list = to_streamed_response_wrapper(
            sim_card_groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            sim_card_groups.delete,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._sim_card_groups.actions)


class AsyncSimCardGroupsResourceWithStreamingResponse:
    def __init__(self, sim_card_groups: AsyncSimCardGroupsResource) -> None:
        self._sim_card_groups = sim_card_groups

        self.create = async_to_streamed_response_wrapper(
            sim_card_groups.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            sim_card_groups.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            sim_card_groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sim_card_groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sim_card_groups.delete,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._sim_card_groups.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .sim_card_group import SimCardGroup

__all__ = ["SimCardGroupRetrieveResponse"]


class SimCardGroupRetrieveResponse(BaseModel):
    data: Optional[SimCardGroup] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["PlanAddStepsToPlanParams", "Step"]


class PlanAddStepsToPlanParams(TypedDict, total=False):
    mission_id: Required[str]

    steps: Required[Iterable[Step]]


class Step(TypedDict, total=False):
    description: Required[str]

    sequence: Required[int]

    step_id: Required[str]

    metadata: Dict[str, object]

    parent_step_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["TestListParams"]


class TestListParams(TypedDict, total=False):
    destination: str
    """Filter tests by destination (phone number, webhook URL, etc.)"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    telnyx_conversation_channel: str
    """Filter tests by communication channel (e.g., 'web_chat', 'sms')"""

    test_suite: str
    """Filter tests by test suite name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .users.users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)

__all__ = ["OrganizationsResource", "AsyncOrganizationsResource"]


class OrganizationsResource(SyncAPIResource):
    @cached_property
    def users(self) -> UsersResource:
        """Operations related to users in your organization"""
        return UsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> OrganizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OrganizationsResourceWithStreamingResponse(self)


class AsyncOrganizationsResource(AsyncAPIResource):
    @cached_property
    def users(self) -> AsyncUsersResource:
        """Operations related to users in your organization"""
        return AsyncUsersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOrganizationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOrganizationsResourceWithStreamingResponse(self)


class OrganizationsResourceWithRawResponse:
    def __init__(self, organizations: OrganizationsResource) -> None:
        self._organizations = organizations

    @cached_property
    def users(self) -> UsersResourceWithRawResponse:
        """Operations related to users in your organization"""
        return UsersResourceWithRawResponse(self._organizations.users)


class AsyncOrganizationsResourceWithRawResponse:
    def __init__(self, organizations: AsyncOrganizationsResource) -> None:
        self._organizations = organizations

    @cached_property
    def users(self) -> AsyncUsersResourceWithRawResponse:
        """Operations related to users in your organization"""
        return AsyncUsersResourceWithRawResponse(self._organizations.users)


class OrganizationsResourceWithStreamingResponse:
    def __init__(self, organizations: OrganizationsResource) -> None:
        self._organizations = organizations

    @cached_property
    def users(self) -> UsersResourceWithStreamingResponse:
        """Operations related to users in your organization"""
        return UsersResourceWithStreamingResponse(self._organizations.users)


class AsyncOrganizationsResourceWithStreamingResponse:
    def __init__(self, organizations: AsyncOrganizationsResource) -> None:
        self._organizations = organizations

    @cached_property
    def users(self) -> AsyncUsersResourceWithStreamingResponse:
        """Operations related to users in your organization"""
        return AsyncUsersResourceWithStreamingResponse(self._organizations.users)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AudioVisualizerConfigParam"]


class AudioVisualizerConfigParam(TypedDict, total=False):
    color: Literal["verdant", "twilight", "bloom", "mystic", "flare", "glacier"]
    """The color theme for the audio visualizer."""

    preset: str
    """The preset style for the audio visualizer."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingEventMessagingChangedPayload", "Payload", "PayloadMessaging"]


class PayloadMessaging(BaseModel):
    """The messaging portability status of the porting order."""

    enable_messaging: Optional[bool] = None
    """
    Indicates whether Telnyx will port messaging capabilities from the losing
    carrier. If false, any messaging capabilities will stay with their current
    provider.
    """

    messaging_capable: Optional[bool] = None
    """Indicates whether the porting order is messaging capable."""

    messaging_port_completed: Optional[bool] = None
    """Indicates whether the messaging port is completed."""

    messaging_port_status: Optional[
        Literal["not_applicable", "pending", "activating", "exception", "canceled", "partial_port_complete", "ported"]
    ] = None
    """Indicates the messaging port status of the porting order."""


class Payload(BaseModel):
    """The webhook payload for the porting_order.messaging_changed event"""

    id: Optional[str] = None
    """Identifies the porting order that was moved."""

    customer_reference: Optional[str] = None
    """Identifies the customer reference associated with the porting order."""

    messaging: Optional[PayloadMessaging] = None
    """The messaging portability status of the porting order."""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""


class PortingEventMessagingChangedPayload(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["porting_order.messaging_changed"]] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None
    """The webhook payload for the porting_order.messaging_changed event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.phone_numbers import (
    action_enable_emergency_params,
    action_verify_ownership_params,
    action_change_bundle_status_params,
)
from ...types.phone_numbers.action_enable_emergency_response import ActionEnableEmergencyResponse
from ...types.phone_numbers.action_verify_ownership_response import ActionVerifyOwnershipResponse
from ...types.phone_numbers.action_change_bundle_status_response import ActionChangeBundleStatusResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    """Configure your phone numbers"""

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def change_bundle_status(
        self,
        id: str,
        *,
        bundle_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionChangeBundleStatusResponse:
        """
        Change the bundle status for a phone number (set to being in a bundle or remove
        from a bundle)

        Args:
          bundle_id: The new bundle_id setting for the number. If you are assigning the number to a
              bundle, this is the unique ID of the bundle you wish to use. If you are removing
              the number from a bundle, this must be null. You cannot assign a number from one
              bundle to another directly. You must first remove it from a bundle, and then
              assign it to a new bundle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/phone_numbers/{id}/actions/bundle_status_change",
            body=maybe_transform(
                {"bundle_id": bundle_id}, action_change_bundle_status_params.ActionChangeBundleStatusParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionChangeBundleStatusResponse,
        )

    def enable_emergency(
        self,
        id: str,
        *,
        emergency_address_id: str,
        emergency_enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEnableEmergencyResponse:
        """
        Enable emergency for a phone number

        Args:
          emergency_address_id: Identifies the address to be used with emergency services.

          emergency_enabled: Indicates whether to enable emergency services on this number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/phone_numbers/{id}/actions/enable_emergency",
            body=maybe_transform(
                {
                    "emergency_address_id": emergency_address_id,
                    "emergency_enabled": emergency_enabled,
                },
                action_enable_emergency_params.ActionEnableEmergencyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEnableEmergencyResponse,
        )

    def verify_ownership(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionVerifyOwnershipResponse:
        """
        Verifies ownership of the provided phone numbers and returns a mapping of
        numbers to their IDs, plus a list of numbers not found in the account.

        Args:
          phone_numbers: Array of phone numbers to verify ownership for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/phone_numbers/actions/verify_ownership",
            body=maybe_transform(
                {"phone_numbers": phone_numbers}, action_verify_ownership_params.ActionVerifyOwnershipParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionVerifyOwnershipResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    """Configure your phone numbers"""

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def change_bundle_status(
        self,
        id: str,
        *,
        bundle_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionChangeBundleStatusResponse:
        """
        Change the bundle status for a phone number (set to being in a bundle or remove
        from a bundle)

        Args:
          bundle_id: The new bundle_id setting for the number. If you are assigning the number to a
              bundle, this is the unique ID of the bundle you wish to use. If you are removing
              the number from a bundle, this must be null. You cannot assign a number from one
              bundle to another directly. You must first remove it from a bundle, and then
              assign it to a new bundle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/phone_numbers/{id}/actions/bundle_status_change",
            body=await async_maybe_transform(
                {"bundle_id": bundle_id}, action_change_bundle_status_params.ActionChangeBundleStatusParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionChangeBundleStatusResponse,
        )

    async def enable_emergency(
        self,
        id: str,
        *,
        emergency_address_id: str,
        emergency_enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEnableEmergencyResponse:
        """
        Enable emergency for a phone number

        Args:
          emergency_address_id: Identifies the address to be used with emergency services.

          emergency_enabled: Indicates whether to enable emergency services on this number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/phone_numbers/{id}/actions/enable_emergency",
            body=await async_maybe_transform(
                {
                    "emergency_address_id": emergency_address_id,
                    "emergency_enabled": emergency_enabled,
                },
                action_enable_emergency_params.ActionEnableEmergencyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEnableEmergencyResponse,
        )

    async def verify_ownership(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionVerifyOwnershipResponse:
        """
        Verifies ownership of the provided phone numbers and returns a mapping of
        numbers to their IDs, plus a list of numbers not found in the account.

        Args:
          phone_numbers: Array of phone numbers to verify ownership for

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/phone_numbers/actions/verify_ownership",
            body=await async_maybe_transform(
                {"phone_numbers": phone_numbers}, action_verify_ownership_params.ActionVerifyOwnershipParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionVerifyOwnershipResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.change_bundle_status = to_raw_response_wrapper(
            actions.change_bundle_status,
        )
        self.enable_emergency = to_raw_response_wrapper(
            actions.enable_emergency,
        )
        self.verify_ownership = to_raw_response_wrapper(
            actions.verify_ownership,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.change_bundle_status = async_to_raw_response_wrapper(
            actions.change_bundle_status,
        )
        self.enable_emergency = async_to_raw_response_wrapper(
            actions.enable_emergency,
        )
        self.verify_ownership = async_to_raw_response_wrapper(
            actions.verify_ownership,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.change_bundle_status = to_streamed_response_wrapper(
            actions.change_bundle_status,
        )
        self.enable_emergency = to_streamed_response_wrapper(
            actions.enable_emergency,
        )
        self.verify_ownership = to_streamed_response_wrapper(
            actions.verify_ownership,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.change_bundle_status = async_to_streamed_response_wrapper(
            actions.change_bundle_status,
        )
        self.enable_emergency = async_to_streamed_response_wrapper(
            actions.enable_emergency,
        )
        self.verify_ownership = async_to_streamed_response_wrapper(
            actions.verify_ownership,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.texml.accounts.calls import recording_recording_sid_json_params
from .....types.texml.accounts.calls.recording_recording_sid_json_response import RecordingRecordingSidJsonResponse

__all__ = ["RecordingsResource", "AsyncRecordingsResource"]


class RecordingsResource(SyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> RecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RecordingsResourceWithStreamingResponse(self)

    def recording_sid_json(
        self,
        recording_sid: str,
        *,
        account_sid: str,
        call_sid: str,
        status: Literal["in-progress", "paused", "stopped"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingRecordingSidJsonResponse:
        """
        Updates recording resource for particular call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        if not recording_sid:
            raise ValueError(f"Expected a non-empty value for `recording_sid` but received {recording_sid!r}")
        return self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings/{recording_sid}.json",
            body=maybe_transform(
                {"status": status}, recording_recording_sid_json_params.RecordingRecordingSidJsonParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingRecordingSidJsonResponse,
        )


class AsyncRecordingsResource(AsyncAPIResource):
    """TeXML REST Commands"""

    @cached_property
    def with_raw_response(self) -> AsyncRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRecordingsResourceWithStreamingResponse(self)

    async def recording_sid_json(
        self,
        recording_sid: str,
        *,
        account_sid: str,
        call_sid: str,
        status: Literal["in-progress", "paused", "stopped"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingRecordingSidJsonResponse:
        """
        Updates recording resource for particular call.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_sid:
            raise ValueError(f"Expected a non-empty value for `account_sid` but received {account_sid!r}")
        if not call_sid:
            raise ValueError(f"Expected a non-empty value for `call_sid` but received {call_sid!r}")
        if not recording_sid:
            raise ValueError(f"Expected a non-empty value for `recording_sid` but received {recording_sid!r}")
        return await self._post(
            f"/texml/Accounts/{account_sid}/Calls/{call_sid}/Recordings/{recording_sid}.json",
            body=await async_maybe_transform(
                {"status": status}, recording_recording_sid_json_params.RecordingRecordingSidJsonParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingRecordingSidJsonResponse,
        )


class RecordingsResourceWithRawResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.recording_sid_json = to_raw_response_wrapper(
            recordings.recording_sid_json,
        )


class AsyncRecordingsResourceWithRawResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.recording_sid_json = async_to_raw_response_wrapper(
            recordings.recording_sid_json,
        )


class RecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.recording_sid_json = to_streamed_response_wrapper(
            recordings.recording_sid_json,
        )


class AsyncRecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.recording_sid_json = async_to_streamed_response_wrapper(
            recordings.recording_sid_json,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DynamicEmergencyEndpointCreateParams"]


class DynamicEmergencyEndpointCreateParams(TypedDict, total=False):
    callback_number: Required[str]

    caller_name: Required[str]

    dynamic_emergency_address_id: Required[str]
    """An id of a currently active dynamic emergency location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SimCardOrderPreviewPreviewParams"]


class SimCardOrderPreviewPreviewParams(TypedDict, total=False):
    address_id: Required[str]
    """Uniquely identifies the address for the order."""

    quantity: Required[int]
    """
    The amount of SIM cards that the user would like to purchase in the SIM card
    order.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["WhatsappLocation"]


class WhatsappLocation(BaseModel):
    address: Optional[str] = None

    latitude: Optional[str] = None

    longitude: Optional[str] = None

    name: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .user_bundle import UserBundle

__all__ = ["UserBundleRetrieveResponse"]


class UserBundleRetrieveResponse(BaseModel):
    data: UserBundle



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OAuthIntrospectParams"]


class OAuthIntrospectParams(TypedDict, total=False):
    token: Required[str]
    """The token to introspect"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .sim_card_action import SimCardAction as SimCardAction
from .action_list_params import ActionListParams as ActionListParams
from .bulk_sim_card_action import BulkSimCardAction as BulkSimCardAction
from .action_enable_response import ActionEnableResponse as ActionEnableResponse
from .action_disable_response import ActionDisableResponse as ActionDisableResponse
from .action_retrieve_response import ActionRetrieveResponse as ActionRetrieveResponse
from .action_set_public_ip_params import ActionSetPublicIPParams as ActionSetPublicIPParams
from .action_set_standby_response import ActionSetStandbyResponse as ActionSetStandbyResponse
from .action_set_public_ip_response import ActionSetPublicIPResponse as ActionSetPublicIPResponse
from .action_bulk_enable_voice_params import ActionBulkEnableVoiceParams as ActionBulkEnableVoiceParams
from .action_bulk_disable_voice_params import ActionBulkDisableVoiceParams as ActionBulkDisableVoiceParams
from .action_remove_public_ip_response import ActionRemovePublicIPResponse as ActionRemovePublicIPResponse
from .action_bulk_enable_voice_response import ActionBulkEnableVoiceResponse as ActionBulkEnableVoiceResponse
from .action_bulk_set_public_ips_params import ActionBulkSetPublicIPsParams as ActionBulkSetPublicIPsParams
from .action_bulk_disable_voice_response import ActionBulkDisableVoiceResponse as ActionBulkDisableVoiceResponse
from .action_bulk_set_public_ips_response import ActionBulkSetPublicIPsResponse as ActionBulkSetPublicIPsResponse
from .action_validate_registration_codes_params import (
    ActionValidateRegistrationCodesParams as ActionValidateRegistrationCodesParams,
)
from .action_validate_registration_codes_response import (
    ActionValidateRegistrationCodesResponse as ActionValidateRegistrationCodesResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_refer_started import CallReferStarted

__all__ = ["CallReferStartedWebhookEvent"]


class CallReferStartedWebhookEvent(BaseModel):
    data: Optional[CallReferStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ClusterComputeParams"]


class ClusterComputeParams(TypedDict, total=False):
    bucket: Required[str]
    """The embedded storage bucket to compute the clusters from.

    The bucket must already be
    [embedded](https://developers.telnyx.com/api-reference/embeddings/embed-documents).
    """

    files: SequenceNotStr[str]
    """Array of files to filter which are included."""

    min_cluster_size: int
    """Smallest number of related text chunks to qualify as a cluster.

    Top-level clusters should be thought of as identifying broad themes in your
    data.
    """

    min_subcluster_size: int
    """Smallest number of related text chunks to qualify as a sub-cluster.

    Sub-clusters should be thought of as identifying more specific topics within a
    broader theme.
    """

    prefix: str
    """Prefix to filter whcih files in the buckets are included."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .conversation import Conversation

__all__ = ["ConversationListResponse"]


class ConversationListResponse(BaseModel):
    data: List[Conversation]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MobileNetworkOperatorListParams", "Filter", "FilterName"]


class MobileNetworkOperatorListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for mobile network operators (deepObject style).

    Originally: filter[name][starts_with], filter[name][contains],
    filter[name][ends_with], filter[country_code], filter[mcc], filter[mnc],
    filter[tadig], filter[network_preferences_enabled]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterName(TypedDict, total=False):
    """Advanced name filtering operations"""

    contains: str
    """Filter by name containing match."""

    ends_with: str
    """Filter by name ending with."""

    starts_with: str
    """Filter by name starting with."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter for mobile network operators (deepObject style).

    Originally: filter[name][starts_with], filter[name][contains], filter[name][ends_with], filter[country_code], filter[mcc], filter[mnc], filter[tadig], filter[network_preferences_enabled]
    """

    country_code: str
    """Filter by exact country_code."""

    mcc: str
    """Filter by exact MCC."""

    mnc: str
    """Filter by exact MNC."""

    name: FilterName
    """Advanced name filtering operations"""

    network_preferences_enabled: bool
    """Filter by network_preferences_enabled."""

    tadig: str
    """Filter by exact TADIG."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TelnyxAgentLinkParams"]


class TelnyxAgentLinkParams(TypedDict, total=False):
    mission_id: Required[str]

    telnyx_agent_id: Required[str]
    """The Telnyx AI agent ID to link"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = ["StreamServerEvent", "AudioChunkFrame", "FinalFrame", "ErrorFrame"]


class AudioChunkFrame(BaseModel):
    """Server-to-client frame containing a base64-encoded audio chunk."""

    audio: Optional[str] = None
    """Base64-encoded audio data.

    May be `null` for providers that use `drop_concatenated_audio` mode (Telnyx
    Natural/NaturalHD, Rime, Minimax, MurfAI, Resemble) — in that case only streamed
    chunks carry audio.
    """

    cached: Optional[bool] = None
    """Whether this audio was served from cache."""

    is_final: Optional[bool] = FieldInfo(alias="isFinal", default=None)
    """Always `false` for audio chunk frames."""

    text: Optional[str] = None
    """The text segment that this audio chunk corresponds to."""

    time_to_first_audio_frame_ms: Optional[int] = FieldInfo(alias="timeToFirstAudioFrameMs", default=None)
    """Milliseconds from the start-of-speech request to the first audio frame.

    Only present on the first audio chunk of a synthesis request.
    """

    type: Optional[Literal["audio_chunk"]] = None
    """Frame type identifier."""


class FinalFrame(BaseModel):
    """Server-to-client frame indicating synthesis is complete for the current text."""

    audio: None = None
    """Always `null` for the final frame."""

    is_final: Optional[Literal[True]] = FieldInfo(alias="isFinal", default=None)
    """Always `true`."""

    text: Optional[str] = None
    """Empty string."""

    time_to_first_audio_frame_ms: Optional[int] = FieldInfo(alias="timeToFirstAudioFrameMs", default=None)
    """Present if this was the first response frame."""

    type: Optional[Literal["final"]] = None
    """Frame type identifier."""


class ErrorFrame(BaseModel):
    """Server-to-client frame indicating an error during synthesis.

    The connection is closed shortly after.
    """

    error: Optional[str] = None
    """Error message describing what went wrong."""

    type: Optional[Literal["error"]] = None
    """Frame type identifier."""


StreamServerEvent: TypeAlias = Annotated[
    Union[AudioChunkFrame, FinalFrame, ErrorFrame], PropertyInfo(discriminator="type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .event_data import EventData
from ....._models import BaseModel

__all__ = ["EventGetEventDetailsResponse"]


class EventGetEventDetailsResponse(BaseModel):
    data: EventData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .rcs_suggestion import RcsSuggestion
from .rcs_card_content import RcsCardContent
from .rcs_content_info import RcsContentInfo

__all__ = [
    "RcsAgentMessage",
    "ContentMessage",
    "ContentMessageRichCard",
    "ContentMessageRichCardCarouselCard",
    "ContentMessageRichCardStandaloneCard",
    "Event",
]


class ContentMessageRichCardCarouselCard(BaseModel):
    """Carousel of cards."""

    card_contents: List[RcsCardContent]
    """The list of contents for each card in the carousel.

    A carousel can have a minimum of 2 cards and a maximum 10 cards.
    """

    card_width: Literal["CARD_WIDTH_UNSPECIFIED", "SMALL", "MEDIUM"]
    """The width of the cards in the carousel."""


class ContentMessageRichCardStandaloneCard(BaseModel):
    """Standalone card"""

    card_content: RcsCardContent

    card_orientation: Literal["CARD_ORIENTATION_UNSPECIFIED", "HORIZONTAL", "VERTICAL"]
    """Orientation of the card."""

    thumbnail_image_alignment: Literal["THUMBNAIL_IMAGE_ALIGNMENT_UNSPECIFIED", "LEFT", "RIGHT"]
    """Image preview alignment for standalone cards with horizontal layout."""


class ContentMessageRichCard(BaseModel):
    carousel_card: Optional[ContentMessageRichCardCarouselCard] = None
    """Carousel of cards."""

    standalone_card: Optional[ContentMessageRichCardStandaloneCard] = None
    """Standalone card"""


class ContentMessage(BaseModel):
    content_info: Optional[RcsContentInfo] = None

    rich_card: Optional[ContentMessageRichCard] = None

    suggestions: Optional[List[RcsSuggestion]] = None
    """List of suggested actions and replies"""

    text: Optional[str] = None
    """Text (maximum 3072 characters)"""


class Event(BaseModel):
    """RCS Event to send to the recipient"""

    event_type: Optional[Literal["TYPE_UNSPECIFIED", "IS_TYPING", "READ"]] = None


class RcsAgentMessage(BaseModel):
    content_message: Optional[ContentMessage] = None

    event: Optional[Event] = None
    """RCS Event to send to the recipient"""

    expire_time: Optional[datetime] = None
    """Timestamp in UTC of when this message is considered expired"""

    ttl: Optional[str] = None
    """Duration in seconds ending with 's'"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_conversation_ended import CallConversationEnded

__all__ = ["CallConversationEndedWebhookEvent"]


class CallConversationEndedWebhookEvent(BaseModel):
    data: Optional[CallConversationEnded] = None
